@@ -369,53 +369,6 @@ ItclEnsembleSubCmd(
     return result;
 }
 
-
-/*
- * ------------------------------------------------------------------------
- *  ItclTraceUnsetVar()
- * ------------------------------------------------------------------------
- */
-
-char *
-ItclTraceUnsetVar(
-    ClientData clientData,
-    Tcl_Interp *interp,
-    const char *name1,
-    const char *name2,
-    int flags)
-{
-#ifdef NOTDEF
-    IctlVarTraceInfo *tracePtr;
-    Tcl_HashEntry *hPtr;
-#endif
-
-    if (name2 != NULL) {
-        /* unsetting of an array element nothing to do */
-	return NULL;
-    }
-    /* also when unsetting variables, they stay alive until the class
-     * or object is teared down!!
-     */
-#ifdef NOTDEF
-    tracePtr = (IctlVarTraceInfo *)clientData;
-    if (tracePtr->flags & ITCL_TRACE_CLASS) {
-        hPtr = Tcl_FindHashEntry(&tracePtr->iclsPtr->classCommons,
-	        (char *)tracePtr->ivPtr);
-	if (hPtr != NULL) {
-	    Tcl_DeleteHashEntry(hPtr);
-	}
-    }
-    if (tracePtr->flags & ITCL_TRACE_OBJECT) {
-        hPtr = Tcl_FindHashEntry(&tracePtr->ioPtr->objectVariables,
-	        (char *)tracePtr->ivPtr);
-	if (hPtr != NULL) {
-	    Tcl_DeleteHashEntry(hPtr);
-	}
-    }
-    ckfree((char *)tracePtr);
-#endif
-    return NULL;
-}
 
 /*
  * ------------------------------------------------------------------------
@@ -481,6 +434,7 @@ AddDictEntry(
     }
     keyPtr = Tcl_NewStringObj(keyStr, -1);
     if (Tcl_DictObjPut(interp, dictPtr, keyPtr, valuePtr) != TCL_OK) {
+	Tcl_DecrRefCount(keyPtr);
         return TCL_ERROR;
     }
     return TCL_OK;
@@ -742,8 +696,8 @@ ItclAddObjectsDictInfo(
     }
     objPtr = Tcl_NewObj();
     Tcl_GetCommandFullName(interp, ioPtr->accessCmd, objPtr);
-    Tcl_IncrRefCount(objPtr);
     if (AddDictEntry(interp, valuePtr2, "-command", objPtr) != TCL_OK) {
+	Tcl_DecrRefCount(objPtr);
         return TCL_ERROR;
     }
     keyPtr = ioPtr->namePtr;
@@ -751,9 +705,11 @@ ItclAddObjectsDictInfo(
         return TCL_ERROR;
     }
     if (newValue1) {
-        if (Tcl_DictObjPut(interp, dictPtr, keyPtr1, valuePtr1) != TCL_OK) {
-            return TCL_ERROR;
-        }
+	/* Cannot fail. Screened non-dicts earlier. */
+        Tcl_DictObjPut(interp, dictPtr, keyPtr1, valuePtr1);
+    } else {
+	/* Don't leak the key val... */
+	Tcl_DecrRefCount(keyPtr1);
     }
     Tcl_SetVar2Ex(interp, ITCL_NAMESPACE"::internal::dicts::objects",
             NULL, dictPtr, 0);
@@ -785,28 +741,36 @@ ItclDeleteObjectsDictInfo(
     }
     keyPtr1 = Tcl_NewStringObj("instances", -1);
     if (Tcl_DictObjGet(interp, dictPtr, keyPtr1, &valuePtr) != TCL_OK) {
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_ERROR;
     }
     if (valuePtr == NULL) {
 	/* looks like no object has been registered yet
 	 * so ignore and return OK */
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_OK;
     }
     keyPtr = ioPtr->namePtr;
     if (Tcl_DictObjGet(interp, valuePtr, keyPtr, &valuePtr1) != TCL_OK) {
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_ERROR;
     }
     if (valuePtr1 == NULL) {
 	/* looks like the object has not been constructed successfully
 	 * so ignore and return OK */
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_OK;
     }
     if (Tcl_DictObjRemove(interp, valuePtr, keyPtr) != TCL_OK) {
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_ERROR;
     }
     if (Tcl_DictObjPut(interp, dictPtr, keyPtr1, valuePtr) != TCL_OK) {
+	/* This is very likely impossible. non-dict already screened. */
+	Tcl_DecrRefCount(keyPtr1);
         return TCL_ERROR;
     }
+    Tcl_DecrRefCount(keyPtr1);
     Tcl_SetVar2Ex(interp, ITCL_NAMESPACE"::internal::dicts::objects",
             NULL, dictPtr, 0);
     return TCL_OK;
