@@ -38,6 +38,9 @@ typedef struct ItclResolvedVarInfo {
 
 static const char * special_resolve_vars[] = {
    "this",
+   "win",
+   "type",
+   "selfns",
    "self",
    "itk_option",
    "itk_interior",
@@ -66,7 +69,7 @@ static Tcl_Var ItclClassRuntimeVarResolver(
 int
 Itcl_ClassCmdResolver(
     Tcl_Interp *interp,		/* current interpreter */
-    CONST char* name,		/* name of the command being accessed */
+    const char* name,		/* name of the command being accessed */
     Tcl_Namespace *nsPtr,	/* namespace performing the resolution */
     int flags,			/* TCL_LEAVE_ERR_MSG => leave error messages
 				 *   in interp if anything goes wrong */
@@ -78,7 +81,6 @@ Itcl_ClassCmdResolver(
     ItclClass *iclsPtr;
     ItclObjectInfo *infoPtr;
     ItclMemberFunc *imPtr;
-    ItclDelegatedFunction *idmPtr;
     int inOptionHandling;
     int isCmdDeleted;
 
@@ -106,7 +108,6 @@ Itcl_ClassCmdResolver(
 	    hPtr = Tcl_FindHashEntry(&iclsPtr->delegatedFunctions,
 	            (char *)namePtr);
 	    if (hPtr != NULL) {
-	        idmPtr = Tcl_GetHashValue(hPtr);
                 objPtr = Tcl_NewStringObj("unknown", -1);
                 hPtr = Tcl_FindHashEntry(&iclsPtr->resolveCmds, (char *)objPtr);
                 Tcl_DecrRefCount(objPtr);
@@ -246,7 +247,7 @@ isCmdDeleted = 0;
 int
 Itcl_ClassVarResolver(
     Tcl_Interp *interp,       /* current interpreter */
-    CONST char* name,	      /* name of the variable being accessed */
+    const char* name,	      /* name of the variable being accessed */
     Tcl_Namespace *nsPtr,     /* namespace performing the resolution */
     int flags,                /* TCL_LEAVE_ERR_MSG => leave error messages
                                *   in interp if anything goes wrong */
@@ -263,10 +264,6 @@ Itcl_ClassVarResolver(
     int found;
     const char **cp;
 
-    Tcl_Namespace *upNsPtr;
-    upNsPtr = Itcl_GetUplevelNamespace(interp, 1);
-    assert(Itcl_IsClassNamespace(nsPtr));
-
     contextIoPtr = NULL;
     /*
      *  If this is a global variable, handle it in the usual
@@ -324,6 +321,7 @@ Itcl_ClassVarResolver(
      *  find the object context,
      */
 
+    hPtr = NULL;
     idx = Itcl_GetStackSize(&infoPtr->contextStack) - 1;
     start_idx = idx;
     while (idx >= 0) {
@@ -367,12 +365,6 @@ Itcl_ClassVarResolver(
         hPtr = Tcl_FindHashEntry(&infoPtr->objects, (char *)contextIoPtr);
         if (hPtr == NULL) {
             continue;
-#ifdef NOTDEF
-	    char str[20];
-	    sprintf(str, "%p", contextIoPtr);
-	    Tcl_AppendResult(interp, "contextIoPtr has vanished!!", str, NULL);
-            return TCL_ERROR;
-#endif
         }
         if (contextIoPtr->iclsPtr != vlookup->ivPtr->iclsPtr) {
 	    if (strcmp(Tcl_GetString(vlookup->ivPtr->namePtr), "this") == 0) {
@@ -387,9 +379,6 @@ Itcl_ClassVarResolver(
         hPtr = Tcl_FindHashEntry(&contextIoPtr->objectVariables,
                 (char *)vlookup->ivPtr);
 	/* check if it is a public variable (must be one) if not top_level */
-//	if ((idx != start_idx) && !(vlookup->ivPtr->protection & ITCL_PUBLIC)) {
-//	    continue;
-//	}
 	if (hPtr != NULL) {
 	    break;
 	}
@@ -408,8 +397,8 @@ Itcl_ClassVarResolver(
 
 	Tcl_DStringInit(&buffer);
 	Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
-	Tcl_DStringAppend(&buffer, "::", 2);
-	Tcl_DStringAppend(&buffer, Tcl_GetString(contextIoPtr->namePtr), -1);
+	Tcl_DStringAppend(&buffer,
+		(Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
 	if (vlookup->ivPtr->iclsPtr->nsPtr == NULL) {
 	    /* deletion of class is running */
 	    Tcl_DStringAppend(&buffer,
@@ -431,8 +420,8 @@ Itcl_ClassVarResolver(
 
 	Tcl_DStringInit(&buffer);
 	Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
-	Tcl_DStringAppend(&buffer, "::", 2);
-	Tcl_DStringAppend(&buffer, Tcl_GetString(contextIoPtr->namePtr), -1);
+	Tcl_DStringAppend(&buffer,
+		(Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
 	Tcl_DStringAppend(&buffer, "::itcl_options", -1);
 	varPtr = Itcl_FindNamespaceVar(interp, Tcl_DStringValue(&buffer), NULL, 0);
 	Tcl_DStringFree(&buffer);
@@ -441,6 +430,22 @@ Itcl_ClassVarResolver(
 	    return TCL_OK;
         }
     }
+    if (strcmp(name, "itcl_option_components") == 0) {
+        Tcl_Var varPtr;
+        Tcl_DString buffer;
+
+	Tcl_DStringInit(&buffer);
+	Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
+	Tcl_DStringAppend(&buffer,
+		(Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
+	Tcl_DStringAppend(&buffer, "::itcl_option_components", -1);
+	varPtr = Itcl_FindNamespaceVar(interp, Tcl_DStringValue(&buffer), NULL, 0);
+	Tcl_DStringFree(&buffer);
+        if (varPtr != NULL) {
+            *rPtr = varPtr;
+	    return TCL_OK;
+        }
+    }
     if (hPtr != NULL) {
         *rPtr = Tcl_GetHashValue(hPtr);
         return TCL_OK;
@@ -471,7 +476,7 @@ Itcl_ClassVarResolver(
 int
 Itcl_ClassCompiledVarResolver(
     Tcl_Interp *interp,         /* current interpreter */
-    CONST char* name,           /* name of the variable being accessed */
+    const char* name,           /* name of the variable being accessed */
     int length,                 /* number of characters in name */
     Tcl_Namespace *nsPtr,       /* namespace performing the resolution */
     Tcl_ResolvedVarInfo **rPtr) /* returns: info that makes it possible to
@@ -484,8 +489,6 @@ Itcl_ClassCompiledVarResolver(
     char *buffer;
     char storage[64];
 
-    assert(Itcl_IsClassNamespace(nsPtr));
-
     infoPtr = (ItclObjectInfo *)Tcl_GetAssocData(interp,
                 ITCL_INTERP_DATA, NULL);
     hPtr = Tcl_FindHashEntry(&infoPtr->namespaceClasses, (char *)nsPtr);
@@ -617,9 +620,8 @@ ItclClassRuntimeVarResolver(
 
 	    Tcl_DStringInit(&buffer);
 	    Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
-	    Tcl_DStringAppend(&buffer, "::", 2);
 	    Tcl_DStringAppend(&buffer,
-	            Tcl_GetString(contextIoPtr->namePtr), -1);
+		    (Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
 	    if (vlookup->ivPtr->iclsPtr->nsPtr == NULL) {
 	        Tcl_DStringAppend(&buffer,
 	                Tcl_GetCurrentNamespace(interp)->fullName, -1);
@@ -641,9 +643,8 @@ ItclClassRuntimeVarResolver(
 
 	    Tcl_DStringInit(&buffer);
 	    Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
-	    Tcl_DStringAppend(&buffer, "::", 2);
 	    Tcl_DStringAppend(&buffer,
-	            Tcl_GetString(contextIoPtr->namePtr), -1);
+		    (Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
 	    Tcl_DStringAppend(&buffer, "::itcl_options", -1);
 	    varPtr = Itcl_FindNamespaceVar(interp, Tcl_DStringValue(&buffer),
 	            NULL, 0);
@@ -652,6 +653,23 @@ ItclClassRuntimeVarResolver(
 	        return varPtr;
             }
         }
+        if (strcmp(Tcl_GetString(vlookup->ivPtr->namePtr),
+	        "itcl_option_components") == 0) {
+            Tcl_Var varPtr;
+            Tcl_DString buffer;
+
+	    Tcl_DStringInit(&buffer);
+	    Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
+	    Tcl_DStringAppend(&buffer,
+		    (Tcl_GetObjectNamespace(contextIoPtr->oPtr)->fullName), -1);
+	    Tcl_DStringAppend(&buffer, "::itcl_option_components", -1);
+	    varPtr = Itcl_FindNamespaceVar(interp, Tcl_DStringValue(&buffer),
+	            NULL, 0);
+	    Tcl_DStringFree(&buffer);
+            if (varPtr != NULL) {
+	        return varPtr;
+            }
+        }
         if (hPtr != NULL) {
             return (Tcl_Var)Tcl_GetHashValue(hPtr);
         }
