@@ -974,7 +974,12 @@ Itcl_CanAccess2(
         return 1;
     } else {
         if (protection == ITCL_PRIVATE) {
-            return (iclsPtr->nsPtr == fromNsPtr);
+	    entry = Tcl_FindHashEntry(&iclsPtr->infoPtr->namespaceClasses,
+		fromNsPtr);
+	    if (entry == NULL) {
+		return 0;
+	    }
+	    return (iclsPtr == Tcl_GetHashValue(entry));
         }
     }
 
@@ -986,11 +991,13 @@ Itcl_CanAccess2(
     assert (protection == ITCL_PROTECTED);
 
     if (Itcl_IsClassNamespace(fromNsPtr)) {
-        fromIclsPtr =  (ItclClass*)Tcl_ObjectGetMetadata(fromNsPtr->clientData,
-	        iclsPtr->infoPtr->class_meta_type);
-	if (fromIclsPtr == NULL) {
-	   return 0;
+	entry = Tcl_FindHashEntry(&iclsPtr->infoPtr->namespaceClasses,
+		fromNsPtr);
+	if (entry == NULL) {
+	    return 0;
 	}
+	fromIclsPtr = Tcl_GetHashValue(entry);
+
         entry = Tcl_FindHashEntry(&fromIclsPtr->heritage,
 	        (char*)iclsPtr);
 
